#################################################################################
# Created       : Martin Foster
# Modified      : 04/07/2001
#################################################################################
#
# Comm - Module part of Webchat designed to prompt and handle posts.
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2E 0K9
#
#################################################################################

package Webchat::Comm;						# Pacakace name

use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Date;						# Date hander
use Webchat::Filter;						# Filter system
use Webchat::Option;						# Options handling
use Webchat::Param;						# Parameter control


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Comm
#
# Subroutine primarily charged with displaying input area.   This will also call
# CommHandler which will handle the requests to determin the need for a new
# database entry

sub Comm							# Database, CGI, Parameter list
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $choice   = shift;					# Choice of the matter

	my $java;						# Java options

	my $inline;						# Inline argument list
	my $url = $cgi->url();					# Url of script

	my $puppet;						# Puppet name
	my $color;						# Colour default
	my $size;						# Size default

	my @color;						# Colours
	my @size;						# Size fonts
	my @char;						# Character

	my %realm;						# Realm hash


	#####################
	# Program area

	# Link with Hash
	Webchat::Database::GetHashRealm($Database, \%realm);


	# Populate arrays
	Webchat::Option::Split(\@color, $realm{'OptTextColour'});
	Webchat::Option::Split(\@size,  $realm{'OptTextSize'});
	Webchat::Database::GetListPuppetYours($Database, \@char, $param->{'USER'});
	

	# Cleanup for embed
	delete $param->{'POST'};

	# Define inline and puppet
	$inline = Webchat::Param::EmbedInline($Database, $cgi, %{$param});
	$puppet = $param->{'CHAR'};

	# Retreive size and color
	($size, $color) = Webchat::Database::DataGetFont($Database, $puppet, $color[0]);

	
	# Independent HTML
	if ($choice != 0)
	{

		# Print file and HTML header
		print $cgi->header(-target=>'Comm');
		Webchat::Database::DocumentGetHeaderPlain($Database);
	}

	# With or without javascript as prescribed
	# Determine need for javascript
	($java) = Webchat::Database::DataGetJavascript($Database, $param->{'USER'});

	# Print if javascript is desired
	if ($java eq 'yes')
	{ Webchat::Database::DocumentGetJavascript($Database); }

	# HTML
	# Form
	if ($choice == 1)					# Framed
	{ 
		print $cgi->start_form(-encoding=>'application/x-www-form-urlencoded',
			-method=>'POST',
			-action=>"$url",
			-target=>'Main',
			-name=>'Comm');
	} 
	else							# Dependant/streaming version
	{ print $cgi->start_form(); }
	

	# Embed Main or Comm
	# For basic chat
	if ($choice == 0)
	{  Webchat::Param::EmbedNormal($Database, $cgi, %{$param});  }

	# For framed chat
	elsif ($choice == 1)
	{  Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'MAIN', 'true');
	   print "\t", $cgi->hidden('NUM', $param->{'NUM'}), "\n"; }

	# For streaming or auto-scroll
	elsif ($choice >= 2)
	{   Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'COMM', 'true'); }


	# Top panel and links
	if ($java eq 'yes')
	{
		# Top of table
		print "<CENTER>\n<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=2 BGCOLOR=\"$realm{'SetBGColor'}\" WIDTH=505>\n";

		# Javascript enabled
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">",
		     "$realm{'SetTop'}",
 		     "<A HREF=\"javascript:popup_window('$realm{'LnkPuppet'}$inline',1)\">$realm{'TagLnkPuppet'}</A>", 
		     "<A HREF=\"javascript:popup_window('$realm{'LnkIgnore'}$inline',1)\">$realm{'TagLnkIgnore'}</A>",
		     "<A HREF=\"javascript:popup_window('$realm{'LnkWho'}$inline',1)\">$realm{'TagLnkWho'}</A>",
		     "<A HREF=\"javascript:popup_window('$realm{'LnkPrivate'}$inline',1)\">$realm{'TagLnkPrivate'}</A>",
		     "<A HREF=\"javascript:popup_window('$realm{'LnkRoll'}$inline',1)\">$realm{'TagLnkRoll'}</A>",
	     	     "</TD>\n\t</TR>\n"; 
	}
	else
	{
		# Top of table
		print "<CENTER>\n<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=2 BGCOLOR=\"$realm{'SetBGColor'}\" WIDTH=505>\n";

		# Self containted window
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">",
		      "$realm{'SetTop'}",
		      "<A HREF=\"$realm{'LnkPuppet'}$inline\" TARGET=\"_blank\">$realm{'TagLnkPuppet'}</A>", 
		      "<A HREF=\"$realm{'LnkIgnore'}$inline\" TARGET=\"_blank\">$realm{'TagLnkIgnore'}</A>",
		      "<A HREF=\"$realm{'LnkWho'}$inline\" TARGET=\"_blank\">$realm{'TagLnkWho'}</A>", 
		      "<A HREF=\"$realm{'LnkPrivate'}$inline\" TARGET=\"_blank\">$realm{'TagLnkPrivate'}</A>",
		      "<A HREF=\"$realm{'LnkRoll'}$inline\" TARGET=\"_blank\">$realm{'TagLnkRoll'}</A>",
		      "</TD>\n\t</TR>\n"; 
	}


	# Posting field and side spacers
	print "\t<TR>\n\t\t<TD ROWSPAN=\"6\" WIDTH=\"5\"><FONT COLOR=\"$realm{'SetBGColor'}\">&nbsp;</FONT></TD>\n";
	print "\t\t<TD><TEXTAREA NAME=\"POST\", ROWS=\"3\", COLS=\"45\", WRAP=\"SOFT\"></TEXTAREA></TD>\n";
	print "\t\t<TD>", $cgi->submit($realm{'TxtSubmitButton'}) ,"</TD>\n";
	print "\t\t<TD ROWSPAN=\"6\" WIDTH=\"5\"><FONT COLOR=\"$realm{'SetBGColor'}\">&nbsp;</FONT></TD>\n\t</TR>\n";

	# For basic and framed 
	if ($choice < 2) 
	{
		print "\t<TR>\t\t<TD COLSPAN=\"2\" ALIGN=\"CENTER\"> ", 
		      $cgi->checkbox(-name=>'SCROLLBACK', -label=>$realm{'TxtScrollback'}),
		      " </TD>\n\t</TR>\n";
	}

	# Streaming only
	else
	{
		# Spacing
		print "\t<TR>\n\t\t<TD COLSPAN=\"2\" HEIGHT=\"6\">&nbsp;</TD>\n\t</TR>\n";
	}

	# Spacing
	print "\t<TR>\n\t\t<TD COLSPAN=\"2\" HEIGHT=\"6\">&nbsp;</TD>\n\t</TR>\n";

	# Puppet selection
	print "\t<TR>\n\t\t<TD COLSPAN=\"2\">$realm{'TagPuppet'}", $cgi->popup_menu('CHAR', \@char, $puppet), "</TD>\n\t</TR>\n";

	# Text modifications
	print "\t<TR>\n\t\t<TD COLSPAN=\"2\">$realm{'TagTextColour'}",  $cgi->popup_menu('COLOR', \@color, $color), "</TD>\n\t</TR>\n";
	print "\t<TR>\n\t\t<TD COLSPAN=\"2\">$realm{'TagTextSize'}", $cgi->popup_menu('SIZE', \@size, $size), "</TD>\n\t</TR>\n";


	# Bottom bar
	# Re-Assign Inline
	$inline = Webchat::Param::EmbedInline($Database, $cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});

	# Javascript
	if ($java eq 'yes')
	{
		# Show link
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">",
		      "<A HREF=\"$realm{'LnkPortal'}$inline\" TARGET=\"_blank\">$realm{'TagLnkPortal'}</A>",
		      "<A HREF=\"javascript:popup_window('$realm{'LnkBoard'}$inline',1)\">$realm{'TagLnkBoard'}</A>",
		      "<A HREF=\"javascript:popup_window('$realm{'LnkMail'}$inline&LIST=True',1)\">$realm{'TagLnkMail'}</A>",
		      "</TD>\n\t</TR>\n"; 
	}
	# Lack thereof
	else
	{
		# Show link
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">",
		      "<A HREF=\"$realm{'LnkPortal'}$inline\" TARGET=\"_blank\">$realm{'TagLnkPortal'}</A>",
		      "<A HREF=\"$realm{'LnkBoard'}$inline\" TARGET=\"_blank\">$realm{'TagLnkBoard'}</A>",
		      "<A HREF=\"$realm{'LnkMail'}$inline&LIST=True\" TARGET=\"_blank\">$realm{'TagLnkMail'}</A>",
		      "</TD>\n\t</TR>\n"; 
	}


	# Table Bottom
	print "</TABLE>\n</CENTER>\n";

	# End of form
	print $cgi->end_form(), "\n";

	# Print end of html if needed
	if ($choice != 0)
	{ print $cgi->end_html(); }
}

# Alias for dependency
sub CommDependant
{ Comm(shift, shift, shift, 0); }

# Alias for independnt framed
sub CommIndependant
{ Comm(shift, shift, shift, 1); }

# Alias for independant streaming
sub CommStream
{ Comm(shift, shift, shift, 2); }

# Alias for independant auto-scrolling streaming
sub CommAutoScroll
{ Comm(shift, shift, shift, 3); }


#####################
# DataWrite
#
# Encharged with properly handling information and actually making posts
# once formatted inserting them into the database.

sub DataWrite							# Database, param
{
        #####################
        # Data members
	my $Database = shift;					# Database get
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameters

	my $hex;						# Hex colour
	my %realm;						# Realm hash


        #####################
        # Data members

	# Assign to target if undefined
	$param->{'TARGET'} = (defined($param->{'TARGET'})) ? $param->{'TARGET'} : 'all';

	# Only do following if posts exist
	if ((defined($param->{'POST'})) && ((length($param->{'POST'}) > 1)))
	{
		# Link DBM to hash
		Webchat::Database::GetHashRealm($Database, \%realm);

		# Determin font information
		$hex   = Webchat::Option::Find($realm{'OptTextColour'}, $realm{'OptTextColourHex'}, $param->{'COLOR'});

		# Update if not a dice roll or system wide post
		if (($param->{'TARGET'} ne 'dice roll') && ($param->{'TARGET'} ne $realm{'TxtSystem'}))
		{
			# Update database entry
			$Database->do("UPDATE Puppet 
				SET PuppetTextColour=?, 
				    PuppetTextSize=? 
				WHERE PuppetName=?", {}, 
			 $param->{'COLOR'}, 
		 	 $param->{'SIZE'}, 
			 $param->{'CHAR'});
		}

		# Handle posts
		PostHandler($Database, $cgi, $param, \%realm); 		
	}
	
	# No post simply post for timestamp purposes
	else
	{
		$Database->do("INSERT INTO Post 
			(PostIDNumber,
			 RealmName, 
			 PuppetName,
			 PostTo,
			 PostTimeStamp)
			VALUES(NULL, ?, ?, 'system', ?)", {}, 
		 $param->{'ROOM'}, 
		 $param->{'CHAR'}, 
		 time);

	}
}


#####################
# Post Handler
#
# Depending on rules will handle the formatting of the three styles of posts
# filter language and HTML if needed or specified.

sub PostHandler
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Reference to parameters
	my $realm    = shift;					# Reference to realm

	my $full;						# Full formating
	my $imageless;						# No images
	my $spartan;						# Spartan formatting

	my $bio;						# Bio link
	my $gummy;						# Gummy link
	my $image;						# Image section
	my $inline;						# Inline parameters
	my $tabletop;						# Table top 
	my $tag;						# Tagline
	my $title;						# Title

	my $post;						# Actual post
	my $position = 0;					# Position of post

	my $start;						# Start of table
	my $end;						# End of table

	my $time = Webchat::Date::GetDate(time);		# Date and Time
	my $target = $param->{'TARGET'};			# Target to

	my @realm;						# Realm information
	my @puppet;						# Puppet information
	my @lang;						# Language array
	my @realmpic;						# Ream picture restriction

	my @posts;						# Array of posts


	#####################
	# Program area

	# Retreive puppet and realminformation
	@puppet = Webchat::Database::DataGetPuppetInfo($Database, $param->{'CHAR'});
	@realm  = Webchat::Database::DataGetRealmInfo($Database, $param->{'ROOM'});	

	# Determine colour to use in frames
	if ($target eq 'all')
	{ $tabletop = "WIDTH=\"100%\""; }
	elsif ($target eq $realm->{'TxtSystem'})
	{ $tabletop = "BORDER=\"$realm->{SetBorderSize}\" WIDTH=\"100%\">\n\t<TR BGCOLOR=\"$realm->{SetBackSystem}\">\n\t\t<TD>$realm->{TagHeadSystem}</TD>\n\t<TR"; }
	elsif ($target eq 'dice roll')
	{ $tabletop = "BORDER=\"$realm->{SetBorderSize}\" WIDTH=\"100%\">\n\t<TR BGCOLOR=\"$realm->{SetBackDice}\">\n\t\t<TD>$realm->{TagHeadDice}</TD>\n\t<TR"; }
	elsif ($target ne 'system')
	{ $tabletop = "BORDER=\"$realm->{SetBorderSize}\" WIDTH=\"100%\">\n\t<TR BGCOLOR=\"$realm->{SetBackPrivate}\">\n\t\t<TD>$realm->{TagHeadPrivate}</TD>\n\t<TR"; }

	# Language filtering
	if ($realm[0] eq 'restricted')
	{
		# Retreive words
		# Prepare and execute
		my $Statement = $Database->prepare("SELECT WordSpelling 
			FROM  WordRestriction 
			WHERE RealmName=?");
		$Statement->execute($param->{'ROOM'});

		# Retreive and place in array
		while ($_ = $Statement->fetchrow())
		{ push(@lang, $_); }
		$Statement->finish;

		
		# Run though array of words and filter post
		$_ = @lang;					# Get lenght		
		for (my $count=0; $count < $_; $count++)
		{ $param->{'POST'} =~ s/$lang[$count]/****/gsi; }
	}

	# Bio link
	$gummy = defined($puppet[11]) ? "$realm->{SetGummyRoot}$puppet[11]" : "$realm->{SetGummyRoot}$realm->{OptGummyDefault}";  

	# Inline and format
	$inline = Webchat::Param::EmbedInline($Database, $cgi, 'CHAR', $param->{'CHAR'});
	$bio = "<A HREF=\"$realm->{'LnkBio'}$inline\" TARGET=\"_blank\"><IMG SRC=\"$gummy\" BORDER=\"0\" ALIGN=\"LEFT\"></A>";

	# Format title
	if (defined($puppet[6]))
	{ 
		# Verify if NAME attribute exists
		if ($puppet[6] =~ /NAME/)
		{
			$puppet[6] =~ s/NAME/$param->{'CHAR'}/g;
			$title = $puppet[6];
		}

		# Otherwise prepend name
		else
		{ $title = "$param->{'CHAR'} $title"; }
	}
	

	# Assign chacter name to title
	else
	{ $title = $param->{'CHAR'}; }	

	# Create formatted line
	$title = "$bio <B><FONT SIZE=\"$realm->{'SetTitleSize'}\">$title</FONT></B>";


	# Tagline handling
	$tag = (defined($puppet[2])) ? "(<I>$puppet[2]</I>)" : "";

	# Create tagline
	unless (($target ne 'all') 
            &&  ($target ne 'system')
	    &&  ($target ne 'dice roll')
	    &&  ($target ne $realm->{'TxtSystem'}))
	{ $tag   = "$tag ($time)"; }
	else
	{ $tag   = "$tag ($realm->{'TagPuppetPMFrom'}: $param->{'CHAR'}) ($realm->{'TagPuppetPMTo'}: $param->{'OTARGET'}) ($time)"; }
		

	# Determine colour
	my $hex = Webchat::Option::Find($realm->{'OptTextColour'}, $realm->{'OptTextColourHex'}, $param->{'COLOR'});

	# Image handling
	$image = (defined($puppet[3])) ?  "<A HREF=\"$puppet[3]\" TARGET=\"_TOP\">$realm->{'TagLnkImage'}</A><BR>" : ""; 


	# Filtering of the text
	# Filter HTML if option is selected
	if ($realm[1] eq 'restricted')
	{ $param->{'POST'} =~ s/<[^>]*>//gs; }

	# Standard filtering
	else
	{ ($param->{'POST'}) = Webchat::Filter::SafeHTML($param->{'POST'}); }


	# Large post handling
	# Divide and conquer
	if ((length($param->{'POST'})) > 5000)
	{
		# Cannot allow stray HTML tags
		$param->{'POST'} =~ s/<[^>]*>//gs;

		# Prepend beginning of posts
		$post = substr($param->{'POST'}, 0, 5000);
		push(@posts, "$post ...");

		$position++;

		# Truncate into edible bites
		while (length(substr($param->{'POST'}, ($position * 5000), 5001)) > 5000)
		{
			$post = substr($param->{'POST'}, ($position * 5000), 5000);
			push(@posts, "... $post ...");

			$position++;
		}

		# Append whatever is left
		$post = substr($param->{'POST'}, ($position * 5000), 5000);
		push(@posts, "... $post");
	}
	# Pass as first element of array
	else
	{ push(@posts, "$param->{'POST'}"); }


	# Clear post
	delete $param->{'POST'};


	# Cycle though individual sections
	foreach $post (@posts)
	{

		# Handling of links
		$post      =~ s/$/\n/;
		$post      =~ s/(http:\/\/.*?)[\s]/<A HREF="$1" TARGET="_blank">$1<\/A> /gm;

		# Add colors
		$post = "<FONT SIZE=\"$param->{'SIZE'}\" COLOR=\"$hex\">$post</FONT>";

		# Full formatting
		if (defined($puppet[3]))
		{
			# Retreive image size information
			@realmpic = Webchat::Database::DataGetRealmImageInfo($Database, $param->{'ROOM'});
		
			# Format image format accordingly
			if ((($realmpic[0] != 0) 
			 && ($realmpic[1] != 0)) 
			 && (($puppet[4] > $realmpic[0]) 
			 || ($puppet[5] > $realmpic[1]))) 
			{ $full = "$image$title $tag<BR><BR>$post"; }
			else
			{ $full = "<IMG SRC=\"$puppet[3]\" HEIGHT=\"$puppet[4]\" WIDTH=\"$puppet[5]\" ALIGN=\"left\" BORDER=\"0\">$title $tag<BR><BR>$post"; }
		}
		else
		{ $full = "$title $tag<BR><BR>$post"; }
	

		# Format non imaged format
		$imageless = "$image$title $tag<BR><BR>$post";


		# Create spartan format
		$post =~ s/<[^>]*>//gs;
		$spartan = "$image$title $tag<BR><BR>$post<BR><BR>";


		# Embed all formats in tables
		$start = "<BR>\n<TABLE $tabletop>\n\t<TR>\n\t\t<TD>";
		$end   = "</TD>\n\t</TR>\n</TABLE>\n";

		# Convert \n to a useable character	
		$full      =~ s/\n/<BR>/gs;
		$imageless =~ s/\n/<BR>/gs;
		$spartan   =~ s/\n/<BR>/gs;

		# Assembly
		$full      = "$start$full$end";
		$imageless = "$start$imageless$end";
		$spartan   = "$start$spartan$end";


		# Normal messages
		if (($target eq 'all') || ($target eq 'dice roll'))
		{
			# Insert into database
			$Database->do("INSERT INTO Post 
				(PostIDNumber,
				 RealmName,
				 PuppetName,
				 PostTo,
				 PostTimestamp,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat)		 
				VALUES(NULL, ?, ?, ?, ?, ?, ?, ?)", {}, 
	
			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 'all', 
			 time, 
			 $full, 
			 $imageless, 
			 $spartan);
		}

		# System wide
		elsif ($target eq $realm->{'TxtSystem'})
		{
			#####################
			# Data Members

			my @realms;					# List of realms
			my $realm;					# Specific realm

			# Retreive lists of realms
			Webchat::Database::GetListGlobalRealms($Database, \@realms);

			# Run through each realm
			foreach $realm (@realms)
			{
				# Insert into database
				$Database->do("INSERT INTO Post 
					(PostIDNumber,
					 RealmName,
					 PuppetName,
					 PostTo,
					 PostTimestamp,
					 PostFullFormat,
					 PostImagelessFormat,
					 PostPartialFormat)
					VALUES(NULL, ?, ?, ?, ?, ?, ?, ?)", {}, 
				 $realm, 
				 $param->{'CHAR'}, 
				 'all', 
				 time, 
				 $full, 
				 $imageless, 
				 $spartan);
			}
		}

		# Insert message to be returned to poster
		elsif ($param->{'TARGET'} ne 'system')
		{
			# Send to target
			$Database->do("INSERT INTO Post 
				(PostIDNumber,
				 RealmName,
				 PuppetName,
				 PostTo,
				 PostTimestamp,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat)
				VALUES(NULL, ?, ?, ?, ?, ?, ?, ?)", {}, 
			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 $param->{'TARGET'}, 
			 time, 
			 $full, 
			 $imageless, 
			 $spartan);

			# Send back owner
			$Database->do("INSERT INTO Post 
				(PostIDNumber,
				 RealmName,
				 PuppetName,
				 PostTo,
				 PostTimestamp,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat)
				VALUES(NULL, ?, ?, ?, ?, ?, ?, ?)", {}, 
			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 $param->{'SOURCE'}, 
			 time, 
			 $full, 
			 $imageless, 
			 $spartan);
		}
	}
}

1;								# Return true
